SystemOrganization addCategory: #'AlgebraicDataType-Tests'!
SystemOrganization addCategory: #AlgebraicDataType!

Object subclass: #AlgebraicDataType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType'!

!AlgebraicDataType commentStamp: 'fbs 5/22/2011 20:29' prior: 0!
My subclasses represent algebraic types. My subclasses know how to destructure themselves through implementing #unapply.!

!AlgebraicDataType methodsFor: 'comparing' stamp: 'fbs 5/22/2011 21:14'!
= anObject
	^ (anObject respondsTo: #unapply)
		ifTrue: [self unapply = anObject unapply]
		ifFalse: [anObject isCollection
			ifTrue: [self unapply = anObject]
			ifFalse: [false]]! !

!AlgebraicDataType methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 20:29'!
unapply
	self subclassResponsibility.! !

AlgebraicDataType subclass: #Tree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType'!

Tree class
	instanceVariableNames: 'EmptyTree'!

Tree class
	instanceVariableNames: 'EmptyTree'!

Tree subclass: #Empty
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType'!

!Empty methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 21:05'!
unapply
	^ #(#Empty).! !

Tree subclass: #Leaf
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType'!

!Leaf class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:19'!
value: anObject
	^ self new value: anObject.! !

!Leaf methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 21:16'!
unapply
	^ {#Leaf. value.}! !

!Leaf methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:20'!
value
	^ value.! !

!Leaf methodsFor: 'private' stamp: 'fbs 5/22/2011 20:19'!
value: anObject
	value := anObject.! !

Tree subclass: #Node
	instanceVariableNames: 'left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType'!

!Node class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:21'!
left: aLeftObject 
	^ self left: aLeftObject right: Tree empty.! !

!Node class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:21'!
left: aLeftObject right: aRightObject
	^ self new left: aLeftObject right: aRightObject.! !

!Node class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:21'!
right: aRightObject
	^ self left: Tree empty right: aRightObject.! !

!Node methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:22'!
left
	^ left.! !

!Node methodsFor: 'private' stamp: 'fbs 5/22/2011 20:22'!
left: aLeftObject right: aRightObject
	left := aLeftObject.
	right := aRightObject.! !

!Node methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:22'!
right
	^ right.! !

!Node methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 20:51'!
unapply
	^ {#Node. left. right.}! !

!Tree class methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:27'!
empty
	^ EmptyTree.! !

!Tree class methodsFor: 'class initialization' stamp: 'fbs 5/22/2011 20:28'!
initialize
	EmptyTree := Empty new.! !

TestCase subclass: #EmptyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Tests'!

!EmptyTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:09'!
testEqualsTestsStructuralEquality
	| t1 t2 t3 |
	t1 := Empty new.
	t2 := Empty new.
	t3 := t1 unapply.
	self deny: t1 == t2. "Not the same object."
	self assert: t1 = t2. "But are equal objects."
	self assert: t1 = t3. "Equality means structural equality."! !

!EmptyTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:27'!
testUnapplyShouldDestructure
	self assert: Tree empty unapply = {#Empty}.! !

TestCase subclass: #LeafTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Tests'!

!LeafTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:15'!
testEqualsTestsStructuralEquality
	| t1 t2 t3 |
	t1 := Leaf value: 1.
	t2 := Leaf value: 1.
	t3 := t1 unapply.
	self deny: t1 == t2. "Not the same object."
	self assert: t1 = t2. "But are equal objects."
	self assert: t1 = t3. "Equality means structural equality."! !

!LeafTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:15'!
testUnapplyShouldDestructure
	| destructure tree |
	tree := Leaf value: 1.
	destructure := tree unapply.
	self assert: destructure first = #Leaf.
	self assert: destructure second = tree value.! !

TestCase subclass: #NodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Tests'!

!NodeTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 20:58'!
testEqualsTestsStructuralEquality
	| t1 t2 t3 |
	t1 := Node left: Tree empty.
	t2 := Node left: Tree empty.
	t3 := t1 unapply.
	self deny: t1 == t2. "Not the same object."
	self assert: t1 = t2. "But are equal objects."
	self assert: t1 = t3. "Equality means structural equality."! !

!NodeTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 20:59'!
testUnapplyShouldDestructureLazily
	| destructure tree |
	tree := Node left: (Leaf value: 1) right: (Leaf value: 2).
	destructure := tree unapply.
	self assert: destructure first = #Node.
	self assert: destructure second = tree left.
	self assert: destructure third = tree right.! !
Tree initialize!
